@@ -21,15 +21,12 @@ written to use `self.stdout.write()`,
 - Catherine Devlin, Jan 03 2008 - catherinedevlin.blogspot.com
 
 mercurial repository at http://www.assembla.com/wiki/show/python-cmd2
-CHANGES:
-As of 0.3.0, options should be specified as `optparse` options.  See README.txt.
-flagReader.py options are still supported for backward compatibility
 """
 import cmd, re, os, sys, optparse, subprocess, tempfile, pyparsing, doctest
-import unittest, string, datetime, urllib
+import unittest, string, datetime, urllib, glob
 from code import InteractiveConsole, InteractiveInterpreter, softspace
 from optparse import make_option
-__version__ = '0.5.1'
+__version__ = '0.5.3'
 
 class OptionParser(optparse.OptionParser):
     def exit(self, status=0, msg=None):
@@ -266,10 +263,27 @@ class Cmd(cmd.Cmd):
     nonpythoncommand = 'cmd'
     current_script_dir = None
     reserved_words = []
+    feedback_to_output = False
+    quiet = False
     settable = ['prompt', 'continuation_prompt', 'default_file_name', 'editor',
-                'case_insensitive', 'echo', 'timing', 'abbrev']   
+                'case_insensitive', 'feedback_to_output', 'quiet', 'echo', 'timing', 
+                'abbrev']   
     settable.sort()
     
+    def poutput(self, msg):
+        self.stdout.write(msg)
+        if msg[-1] != '\n':
+            self.stdout.write('\n')
+    def perror(self, errmsg):
+        print str(errmsg)
+    def pfeedback(self, msg):
+        """For printing nonessential feedback.  Can be silenced with `quiet`.
+           Inclusion in redirected output is controlled by `feedback_to_output`."""
+        if not self.quiet:
+            if self.feedback_to_output:
+                self.poutput(msg)
+            else:
+                print msg
     editor = os.environ.get('EDITOR')
     _STOP_AND_EXIT = 2
     if not editor:
@@ -292,16 +306,20 @@ class Cmd(cmd.Cmd):
         })
         
     def do_help(self, arg):
-        funcname = self.func_named(arg)
-        if funcname:
-            fn = getattr(self, funcname)
-            try:
-                fn.optionParser.print_help(file=self.stdout)
-            except AttributeError:
-                cmd.Cmd.do_help(self, funcname[3:])
+        if arg:
+            funcname = self.func_named(arg)
+            if funcname:
+                fn = getattr(self, funcname)
+                try:
+                    fn.optionParser.print_help(file=self.stdout)
+                except AttributeError:
+                    cmd.Cmd.do_help(self, funcname[3:])
+        else:
+            cmd.Cmd.do_help(self, arg)
         
     def __init__(self, *args, **kwargs):        
         cmd.Cmd.__init__(self, *args, **kwargs)
+        self.initial_stdout = sys.stdout
         self.history = History()
         self._init_parser()
         self.pystate = {}
@@ -458,8 +476,8 @@ class Cmd(cmd.Cmd):
           - terminator: ;
         - terminator: ;        
         >>> print c.parser.parseString('multiline command /* with comment in progress;').dump()
-        ['multiline', ' command /* with comment in progress;']
-        - multilineCommand: multiline        
+        ['multiline', ' command']
+        - multilineCommand: multiline
         >>> print c.parser.parseString('multiline command /* with comment complete */ is done;').dump()
         ['multiline', 'command /* with comment complete */ is done', ';', '']
         - args: command /* with comment complete */ is done
@@ -508,12 +526,12 @@ class Cmd(cmd.Cmd):
         #self.multilineParser = self.multilineParser.setResultsName('multilineParser')
         #self.singleLineParser = self.singleLineParser.setResultsName('singleLineParser')
         #self.blankLineTerminationParser = self.blankLineTerminationParser.setResultsName('blankLineTerminatorParser')
-        self.parser = (
+        self.parser = self.prefixParser + (
             stringEnd |
-            self.prefixParser + self.multilineParser |
-            self.prefixParser + self.singleLineParser |
-            self.prefixParser + self.blankLineTerminationParser | 
-            self.prefixParser + self.multilineCommand + pyparsing.SkipTo(stringEnd)
+            self.multilineParser |
+            self.singleLineParser |
+            self.blankLineTerminationParser | 
+            self.multilineCommand + pyparsing.SkipTo(stringEnd)
             )
         self.parser.ignore(pyparsing.quotedString).ignore(self.commentGrammars)
         
@@ -593,14 +611,14 @@ class Cmd(cmd.Cmd):
                                         self.pseudo_raw_input(self.continuation_prompt))                
                 statement = self.parsed(statement)
         except Exception, e:
-            print e
+            self.perror(e)
             return 0
         if statement.parsed.command not in self.excludeFromHistory:
             self.history.append(statement.parsed.raw)
         try:
             (stop, statement) = self.postparsing_precmd(statement)
         except Exception, e:
-            print str(e)
+            self.perror(e)
             return 0
         if stop:
             return self.postparsing_postcmd(stop)
@@ -623,7 +641,7 @@ class Cmd(cmd.Cmd):
                 try:
                     self.stdout = open(os.path.expanduser(statement.parsed.outputTo), mode)                            
                 except OSError, e:
-                    print e
+                    self.perror(e)
                     return self.postparsing_postcmd(stop=0) 
             else:
                 statekeeper = Statekeeper(self, ('stdout',))
@@ -643,9 +661,9 @@ class Cmd(cmd.Cmd):
             timestart = datetime.datetime.now()
             stop = func(statement) 
             if self.timing:
-                print 'Elapsed: %s' % str(datetime.datetime.now() - timestart)
+                self.pfeedback('Elapsed: %s' % str(datetime.datetime.now() - timestart))
         except Exception, e:
-            print e
+            self.perror(e)
         finally:
             if statekeeper:
                 if statement.parsed.output and not statement.parsed.outputTo:
@@ -653,7 +671,7 @@ class Cmd(cmd.Cmd):
                     try:
                         writeToPasteBuffer(self.stdout.read())
                     except Exception, e:
-                        print str(e)
+                        self.perror(e)
                 elif statement.parsed.pipeTo:
                     for result in redirect.communicate():              
                         statekeeper.stdout.write(result or '')                        
@@ -738,7 +756,7 @@ class Cmd(cmd.Cmd):
                 self.stdout.write('%s: %s\n' % (p, str(getattr(self, p))))
                 any_shown = True
         if not any_shown:
-            print "Parameter '%s' not supported (type 'show' for list of parameters)." % param
+            self.perror("Parameter '%s' not supported (type 'show' for list of parameters)." % param)
                 
     def do_quit(self, arg):
         return self._STOP_AND_EXIT
@@ -787,15 +805,7 @@ class Cmd(cmd.Cmd):
     def do_shell(self, arg):
         'execute a command as if at the OS prompt.'
         os.system(arg)
-        
-    def _attempt_py_command(self, arg):
-        try:
-            result = eval(arg, self.pystate)
-            print repr(result)
-        except SyntaxError:
-            exec(arg, self.pystate)       
-        return
-        
+                
     def do_py(self, arg):  
         '''
         py <command>: Executes a Python command.
@@ -874,7 +884,7 @@ class Cmd(cmd.Cmd):
         "set edit (program-name)" or set  EDITOR environment variable
         to control which editing program is used."""
         if not self.editor:
-            print "please use 'set editor' to specify your text editing program of choice."
+            self.perror("Please use 'set editor' to specify your text editing program of choice.")
             return
         filename = self.default_file_name
         if arg:
@@ -907,7 +917,7 @@ class Cmd(cmd.Cmd):
         try:
             args = self.saveparser.parseString(arg)
         except pyparsing.ParseException:
-            print self.do_save.__doc__
+            self.perror(self.do_save.__doc__)
             return
         fname = args.fname or self.default_file_name
         if args.idx == '*':
@@ -920,9 +930,9 @@ class Cmd(cmd.Cmd):
             f = open(os.path.expanduser(fname), 'w')
             f.write(saveme)
             f.close()
-            print 'Saved to %s' % (fname)
+            self.pfeedback('Saved to %s' % (fname))
         except Exception, e:
-            print 'Error saving %s: %s' % (fname, str(e))
+            self.perror('Error saving %s: %s' % (fname, str(e)))
             
     def read_file_or_url(self, fname):
         if isinstance(fname, file):
@@ -962,7 +972,7 @@ class Cmd(cmd.Cmd):
         try:
             target = self.read_file_or_url(targetname)
         except IOError, e:
-            print 'Problem accessing script from %s: \n%s' % (targetname, e)
+            self.perror('Problem accessing script from %s: \n%s' % (targetname, e))
             return
         keepstate = Statekeeper(self, ('stdin','use_rawinput','prompt',
                                        'continuation_prompt','current_script_dir'))
@@ -987,7 +997,7 @@ class Cmd(cmd.Cmd):
         """        
         'run [N]: runs the SQL that was run N commands ago'
         runme = self.last_matching(arg)
-        print runme
+        self.pfeedback(runme)
         if runme:
             runme = self.precmd(runme)
             stop = self.onecmd(runme)
@@ -1107,67 +1117,96 @@ class OutputTrap(Borg):
     def tearDown(self):
         sys.stdout = self.old_stdout
 
+       
 class Cmd2TestCase(unittest.TestCase):
-    '''Subclass this, setting CmdApp and transcriptFileName, to make a unittest.TestCase class
-       that will execute the commands in transcriptFileName and expect the results shown.
+    '''Subclass this, setting CmdApp, to make a unittest.TestCase class
+       that will execute the commands in a transcript file and expect the results shown.
        See example.py'''
     CmdApp = None
-    transcriptFileName = ''
+    def fetchTranscripts(self):
+        self.transcripts = {}
+        for fileset in self.CmdApp.testfiles:
+            for fname in glob.glob(fileset):
+                tfile = open(fname)
+                self.transcripts[fname] = iter(tfile.readlines())
+                tfile.close()
+        if not len(self.transcripts):
+            raise StandardError, "No test files found - nothing to test."
     def setUp(self):
         if self.CmdApp:
             self.outputTrap = OutputTrap()
             self.cmdapp = self.CmdApp()
-            try:
-                tfile = open(os.path.expanduser(self.transcriptFileName))
-                self.transcript = iter(tfile.readlines())
-                tfile.close()
-            except IOError:
-                self.transcript = []
-    def assertEqualEnough(self, got, expected, message):
-        got = got.strip().splitlines()
-        expected = expected.strip().splitlines()
-        self.assertEqual(len(got), len(expected), message)
-        for (linegot, lineexpected) in zip(got, expected):
-            matchme = re.escape(lineexpected.strip()).replace('\\*', '.*').  \
-                      replace('\\ ', ' ')
-            self.assert_(re.match(matchme, linegot.strip()), message)
+            self.fetchTranscripts()
     def testall(self):
         if self.CmdApp:
-            lineNum = 0
-            try:
-                line = self.transcript.next()
-                while True:
-                    while not line.startswith(self.cmdapp.prompt):
-                        line = self.transcript.next()
-                    command = [line[len(self.cmdapp.prompt):]]
-                    line = self.transcript.next()
-                    while line.startswith(self.cmdapp.continuation_prompt):
-                        command.append(line[len(self.cmdapp.continuation_prompt):])
-                        line = self.transcript.next()
-                    command = ''.join(command)
-                    self.cmdapp.onecmd(command)
-                    result = self.outputTrap.read()
-                    if line.startswith(self.cmdapp.prompt):
-                        self.assertEqualEnough(result.strip(), '', 
-                            '\nFile %s, line %d\nCommand was:\n%s\nExpected: (nothing) \nGot:\n%s\n' % 
-                            (self.transcriptFileName, lineNum, command, result))    
-                        continue
-                    expected = []
-                    while not line.startswith(self.cmdapp.prompt):
-                        expected.append(line)
-                        line = self.transcript.next()
-                    expected = ''.join(expected)
-                    self.assertEqualEnough(expected.strip(), result.strip(), 
-                        '\nFile %s, line %d\nCommand was:\n%s\nExpected:\n%s\nGot:\n%s\n' % 
-                        (self.transcriptFileName, lineNum, command, expected, result))    
-                    # this needs to account for a line-by-line strip()ping
-            except StopIteration:
-                pass
-                # catch the final output?
+            its = sorted(self.transcripts.items())
+            for (fname, transcript) in its:
+                self._test_transcript(fname, transcript)
+    regexPattern = pyparsing.QuotedString(quoteChar=r'/', escChar='\\', multiline=True, unquoteResults=True)
+    regexPattern.ignore(pyparsing.cStyleComment)
+    notRegexPattern = pyparsing.Word(pyparsing.printables)
+    notRegexPattern.setParseAction(lambda t: re.escape(t[0]))
+    expectationParser = regexPattern | notRegexPattern
+    endStrippingRegex = re.compile(r'[ \t]*\n')
+    def _test_transcript(self, fname, transcript):
+        lineNum = 0
+        try:
+            line = transcript.next()
+            while True:
+                while not line.startswith(self.cmdapp.prompt):
+                    line = transcript.next()
+                command = [line[len(self.cmdapp.prompt):]]
+                line = transcript.next()
+                while line.startswith(self.cmdapp.continuation_prompt):
+                    command.append(line[len(self.cmdapp.continuation_prompt):])
+                    line = transcript.next()
+                command = ''.join(command)
+                self.cmdapp.onecmd(command)
+                result = self.outputTrap.read().strip()
+                if line.startswith(self.cmdapp.prompt):
+                    message = '\nFile %s, line %d\nCommand was:\n%s\nExpected: (nothing)\nGot:\n%s\n'%\
+                        (fname, lineNum, command, result)     
+                    self.assert_(not(result.strip()), message)
+                    continue
+                expected = []
+                while not line.startswith(self.cmdapp.prompt):
+                    expected.append(line)
+                    line = transcript.next()
+                expected = ''.join(expected).strip()
+                message = '\nFile %s, line %d\nCommand was:\n%s\nExpected:\n%s\nGot:\n%s\n'%\
+                    (fname, lineNum, command, expected, result)      
+                expected = self.expectationParser.transformString(expected)
+                expected = self.endStrippingRegex.sub('\s*\n', expected)
+                self.assert_(re.match(expected, result, re.MULTILINE | re.DOTALL), message)
+        except StopIteration:
+            pass
     def tearDown(self):
         if self.CmdApp:
             self.outputTrap.tearDown()
         
 if __name__ == '__main__':
     doctest.testmod(optionflags = doctest.NORMALIZE_WHITESPACE)
-    #c = Cmd()
+    
+'''
+To make your application transcript-testable, add text like this to your .py file
+(replacing CmdLineApp with your own application class's name).  Then, a cut-and-pasted
+version of a successful session with your application, saved as a text file, can serve
+as a test for future 
+
+Invoke the test later with `python myapplication.py --test mytranscripttestfile.ext`
+Wildcards can be used to test against multiple transcript files.
+
+
+class TestMyAppCase(Cmd2TestCase):
+    CmdApp = CmdLineApp
+parser = optparse.OptionParser()
+parser.add_option('-t', '--test', dest='test', action="store_true", 
+                  help='Test against transcript(s) in FILE (wildcards OK)')
+(callopts, callargs) = parser.parse_args()
+if callopts.test:
+    CmdLineApp.testfiles = callargs
+    sys.argv = [sys.argv[0]] # the --test argument upsets unittest.main()
+    unittest.main()
+else:
+    CmdLineApp().cmdloop()
+'''
\ No newline at end of file
